import struct

import pytest

import cbor2pure._decoder
import cbor2pure._encoder
import cbor2pure._types


@pytest.fixture
def will_overflow():
    """
    Construct an array/string/bytes length which would cause a memory error
    on decode. This should be less than sys.maxsize (the max integer index)
    """
    bit_size = struct.calcsize("P") * 8
    huge_length = 1 << (bit_size - 8)
    return struct.pack("Q", huge_length)


class Module:
    # Mock module class
    pass


@pytest.fixture(scope="session")
def impl():
    # Pure Python implementation only
    module = Module()
    for source in (cbor2pure._types, cbor2pure._encoder, cbor2pure._decoder):
        for name in dir(source):
            setattr(module, name, getattr(source, name))
    return module



from typing import Any

# Import types from cbor2 for compatibility
from cbor2 import CBORDecodeEOF as CBORDecodeEOF
from cbor2 import CBORDecodeError as CBORDecodeError
from cbor2 import CBORDecodeValueError as CBORDecodeValueError
from cbor2 import CBOREncodeError as CBOREncodeError
from cbor2 import CBOREncodeTypeError as CBOREncodeTypeError
from cbor2 import CBOREncodeValueError as CBOREncodeValueError
from cbor2 import CBORError as CBORError
from cbor2 import CBORSimpleValue as CBORSimpleValue
from cbor2 import CBORTag as CBORTag
from cbor2 import FrozenDict as FrozenDict
from cbor2 import undefined as undefined

# Import pure Python encoder/decoder implementations
from ._decoder import CBORDecoder as CBORDecoder
from ._decoder import load as load
from ._decoder import loads as loads
from ._encoder import CBOREncoder as CBOREncoder
from ._encoder import dump as dump
from ._encoder import dumps as dumps
from ._encoder import shareable_encoder as shareable_encoder

# Re-export imports so they look like they live directly in this package
key: str
value: Any
for key, value in list(locals().items()):
    if callable(value) and getattr(value, "__module__", "").startswith("cbor2pure."):
        value.__module__ = __name__



from warnings import warn

# Re-export types from cbor2 for compatibility
from cbor2 import CBORDecodeEOF as CBORDecodeEOF
from cbor2 import CBORDecodeError as CBORDecodeError
from cbor2 import CBORDecodeValueError as CBORDecodeValueError
from cbor2 import CBOREncodeError as CBOREncodeError
from cbor2 import CBOREncodeTypeError as CBOREncodeTypeError
from cbor2 import CBOREncodeValueError as CBOREncodeValueError
from cbor2 import CBORError as CBORError
from cbor2 import CBORSimpleValue as CBORSimpleValue
from cbor2 import CBORTag as CBORTag
from cbor2 import FrozenDict as FrozenDict
from cbor2 import undefined as undefined

warn(
    "The cbor2pure.types module has been deprecated. "
    "Instead import everything directly from cbor2pure."
)



from warnings import warn

from ._decoder import CBORDecoder as CBORDecoder
from ._decoder import load as load
from ._decoder import loads as loads

warn(
    "The cbor2pure.decoder module has been deprecated. "
    "Instead import everything directly from cbor2pure."
)



"""Internal types for cbor2pure. Public types are imported from cbor2."""


class BreakMarkerType:
    """Internal sentinel for indefinite-length decoding."""

    __slots__ = ()

    _instance = None

    def __new__(cls: type["BreakMarkerType"]) -> "BreakMarkerType":
        if cls._instance is None:
            cls._instance = super().__new__(cls)
        return cls._instance

    def __repr__(self) -> str:
        return "break_marker"

    def __bool__(self) -> bool:
        return True


#: Internal sentinel value for indefinite-length decoding
break_marker = BreakMarkerType()



from setuptools import setup

# This is a pure Python package - no C extensions
setup(
    use_scm_version={"version_scheme": "guess-next-dev", "local_scheme": "dirty-tag"},
    setup_requires=["setuptools >= 61", "setuptools_scm >= 6.4"],
)
